/*============================================================================
  Summary:  Demonstrates Joins in SQL Server.
------------------------------------------------------------------------------
  Written by Klaus Aschenbrenner, www.SQLpassion.at

  For more scripts and sample code, check out 
    http://www.SQLpassion.at

  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/

USE AdventureWorks2012
GO

SET STATISTICS IO ON
SET STATISTICS TIME ON
GO

-- =========================
-- Demonstrates Merge Joins
-- =========================

CREATE TABLE Table1
(
	Column1 INT IDENTITY (1, 1) PRIMARY KEY,
	Column2 INT
)
GO

-- Insert 1500 records into Table1
SELECT TOP 1500 IDENTITY(INT, 1, 1) AS n INTO #Nums
FROM
master.dbo.syscolumns sc1

INSERT INTO Table1 (Column2)
SELECT n FROM #nums
DROP TABLE #nums
GO

-- Create another test table
CREATE TABLE Table2
(
	Column1 INT IDENTITY (1, 1) PRIMARY KEY,
	Column2 INT
)
GO

-- Insert 1500 records into Table2
SELECT TOP 1500 IDENTITY(INT, 1, 1) AS n INTO #Nums
FROM
master.dbo.syscolumns sc1

INSERT INTO Table2 (Column2)
SELECT n FROM #nums
DROP TABLE #nums
GO

-- Retrieve the inserted data from both tables.
-- In the 2nd column we have unique values, but the query optimizer doesn't know
-- that fact (no check constraint, no unique index defined)
SELECT * FROM Table1
SELECT * FROM Table2
GO

-- Let's perform a join between both tables:
-- We have to hint SQL Server, so that we get a Merge Join operator in the execution plan.
-- The Query Optimizer introduces an explicit Sort operator to be able to perform the Merge Join.
-- In addition the Merge Join is "Many to Many", because the Query Optimizer has no idea about
-- the unique values in both columns "Column2".
-- The "Many to Many" operation of the Merge Join creates a Worktable in TempDb.
-- Table1 is the outer table, Table2 is the inner table
SELECT a.*, b.* FROM Table1 a
INNER JOIN Table2 b ON a.column2 = b.Column2
OPTION (MERGE JOIN)
GO

-- Now let's create a unique Non-Clustered Index on Column2 of Table2.
CREATE UNIQUE NONCLUSTERED INDEX idx_Col2 ON Table2(Column2)
GO

-- When we now rerun our query, we get a Merge Join operator by default.
-- Because the Query Optimizer now knows that we have unique values in the 2nd table,
-- Table2 is the outer table, and the Merge Join isn't executed as a "Many to Many" operation anymore.
-- The explicit Sort operator on Table2 is also eliminated, because we get a sorted input directly
-- from the accessed Non-Clustered Index.
-- Therefore the query doesn't create a Worktable in TempDb.
SELECT a.*, b.* FROM Table1 a
INNER JOIN Table2 b ON a.column2 = b.Column2
GO

-- When we create another unique Non-Clustered Index on Table1, 
-- we can also eliminate the explicit Sort operator for that table in the execution plan.
CREATE UNIQUE NONCLUSTERED INDEX idx_Col2 ON Table1(Column2)
GO

-- Rerun the query
SELECT a.*, b.* FROM Table1 a
INNER JOIN Table2 b ON a.Column2 = b.Column2
GO

-- Clean up
DROP TABLE Table1
DROP TABLE Table2
GO